/*******************************************************************************
 * (c) Copyright 2016 Microsemi Corporation.  All rights reserved.
 *
 */

#include "hw_platform.h"
#include "drivers/CoreGPIO/core_gpio.h"
#include "drivers/CoreI2C/core_i2c.h"
#include "drivers/CoreUARTapb/core_uart_apb.h"
#include "hal/CortexM1/cortex_nvic.h"
#include "hal.h"
#include "./CMSIS/cortexm1_cfg.h"
#include "./CMSIS/system_cortexm1_cfg.h"

#define DIVISOR 512


/******************************************************************************
 * GPIO instance data. UART instance data. Base addresses
 *****************************************************************************/
gpio_instance_t g_gpio_out, g_gpio_in;
UART_instance_t g_uart0, g_uart1;
i2c_instance_t  g_i2c;


#define BAUD_VALUE                  115200

#define I2C_SFP_CMD_INPUT			0x00
#define I2C_SFP_CMD_OUTPUT			0x01
#define I2C_SFP_CMD_POLARITY		0x02
#define I2C_SFP_CMD_CONF			0x03
#define I2C_WRITE					0
#define I2C_READ					1

/* function prototypes */
void      LED_update(void);
uint8_t * itoa(uint32_t value);
uint8_t   SFP_I2C_CTRL(int cmd, int rw, int ports);
void      SFP_I2C_init(void);
void      SFP_I2C_read_regs(void);
void      Lane_send_byte(uint8_t b);
void      Lane_print_rx(void);

int sema_gpio = 0;

__attribute__((__interrupt__)) void IRQ0_IRQHandler( void )
{
	I2C_isr(&g_i2c);
	HW_set_32bit_reg(NVIC_CLEAR_PEND_REG, NVIC_IRQ_I2C);
}

__attribute__((__interrupt__)) void IRQ1_IRQHandler( void )
{
	sema_gpio = 1;
	GPIO_clear_all_irq_sources(&g_gpio_in, GPIO_get_irq_sources(&g_gpio_in));
	HW_set_32bit_reg(NVIC_CLEAR_PEND_REG, NVIC_IRQ_GPIO_IN);
}
/*-------------------------------------------------------------------------*//**
 * main() function.
 */
int main()
{
	static uint32_t counter;
	static uint32_t timer;

	SystemCoreClockUpdate();

	counter = (SystemCoreClock/DIVISOR);
	timer   = (SystemCoreClock/DIVISOR);

	UART_init(
			&g_uart0,
			UART0_BASE_ADDRESS,
			SYS_M1_CLK_FREQ/((16 * BAUD_VALUE)-1),
			(DATA_8_BITS | NO_PARITY)
	);
	UART_init(
			&g_uart1,
			UART1_BASE_ADDRESS,
			SYS_M1_CLK_FREQ/((16 * BAUD_VALUE)-1),
			(DATA_8_BITS | NO_PARITY)
	);


    /**************************************************************************
     * Initialize the CoreGPIO driver with the base address of the CoreGPIO
     * instance to use and the initial state of the outputs.
     *************************************************************************/
	/* OUTPUTS */
	GPIO_init( &g_gpio_out, COREGPIO_BASE_ADDR, GPIO_APB_32_BITS_BUS );
	/* INPUTS */
	GPIO_init( &g_gpio_in, COREGPIO2_BASE_ADDR, GPIO_APB_32_BITS_BUS );
	GPIO_enable_irq(&g_gpio_in, GPIO_0);
	GPIO_enable_irq(&g_gpio_in, GPIO_1);
	GPIO_enable_irq(&g_gpio_in, GPIO_2);
	GPIO_enable_irq(&g_gpio_in, GPIO_3);

	/* I2C */
	I2C_init(
			&g_i2c,
			I2C0_BASE_ADDRESS,
			0, // master mode -> don't care
			I2C_PCLK_DIV_256
	);

	HW_set_32bit_reg(NVIC_ENABLE_REG, (NVIC_IRQ_I2C | NVIC_IRQ_GPIO_IN));

	UART_polled_tx_string(&g_uart0, (uint8_t *)"UART0, Hello World...\n\r");
	UART_polled_tx_string(&g_uart1, (uint8_t *)"UART1, Hello World...\n\r");

	/*
	 * SFP init
	 * we read out registers of APB_SLAVE before and after initialization
	 * to validate proper function
	 */
	SFP_I2C_read_regs();
	SFP_I2C_init();
	SFP_I2C_read_regs();


	while(1) {
		static int sema_timer_on = 0;
		uint8_t key;

		/*
		 * uart0 rx -> uart1 tx
		 * if a key is hit we send it to our lane, start the timer to produce
		 * a little delay and turn on LED 3
		 */
		if(UART_get_rx(&g_uart0, &key, 1)) {
			UART_send(&g_uart1, &key, 1);
			Lane_send_byte(key);
			sema_timer_on = 1;
			GPIO_set_output(&g_gpio_out, GPIO_2, 1);
		}
		/* uart1 rx -> uart0 tx */
		if(UART_get_rx(&g_uart1, &key, 1)) {
			UART_send(&g_uart0, &key, 1);
		}

		/*
		 * if the timer is started... update it
		 * if the timer expired we reset it, read out the rx of our lane
		 * and turn off LED 3
		 */
		if(sema_timer_on) {
			if(timer-- == 0) {
				sema_timer_on = 0;
				timer = (SystemCoreClock/DIVISOR);
				Lane_print_rx();
				GPIO_set_output(&g_gpio_out, GPIO_2, 0);
			}
		}

		/* push button hit -> counter reset */
		if(sema_gpio) {
			counter = 0;
			sema_gpio = 0;
		}
		/* counter update */
		if(counter-- ==  0) {
			counter = (SystemCoreClock/DIVISOR);
			LED_update();
		}
     }
     return 0;
}

void LED_update(void)
{
	static uint8_t temp = 1;

	temp = temp^1;
	GPIO_set_output(&g_gpio_out, GPIO_0 ,temp);
	GPIO_set_output(&g_gpio_out, GPIO_1 ,!temp);
}
/*
 * simple int to ascii conversion
 */
uint8_t * itoa(uint32_t value)
{
	static uint8_t string[9] = {'0'};
	int i;

	for(i = 7; i >= 0; i--) {
		string[i] = (0x0000000F & value);
		if(string[i] < 0xA)
			string[i] += '0';
		else
			string[i] += ('A' - 0xA);

		value >>= 4;
	}
	string[8] = '\0';

	return string;
}

/*
 * SFP+ Config
 * 	P00:	SFP_RS0			output	0
 * 	P01:	SFP_RS1			input	1
 * 	P02:	SFP_TX_DIS		output	0
 * 	P03:	SFP_MOD			input	1
 * 	P04:	SFP_RX_LOS		input	1
 * 	P05:	SFP_TX_FAULT	input	1
 * 	P06:	nc				input	1
 * 	P07:	nc				input	1
 * 	-> 0xFA
 * I2C-ADDRESS: 1110000 + R/W => 0xE0
 */
uint8_t SFP_I2C_CTRL(int cmd, int rw, int ports)
{
	const uint8_t i2c_addr = 0x70;
	uint8_t data[2] = {0}, rx_data = 0;
	i2c_status_t status;

	data[0] = (uint8_t)cmd;
	data[1] = (uint8_t)ports;

	if(rw == 1) {
		I2C_write_read(&g_i2c, i2c_addr, data, 1, &rx_data, 1, I2C_RELEASE_BUS);
		status = I2C_wait_complete(&g_i2c, 1000);
	} else {
		I2C_write(&g_i2c, i2c_addr, data, 2, I2C_RELEASE_BUS);
		status = I2C_wait_complete(&g_i2c, 1000);
	}
	return rx_data;
}

void SFP_I2C_init(void)
{
	SFP_I2C_CTRL(I2C_SFP_CMD_CONF, I2C_WRITE, 0xFA);
	 // TX_DISABLE = 0, RS0 = 0 -> 1.25Gb/s
	SFP_I2C_CTRL(I2C_SFP_CMD_OUTPUT, I2C_WRITE, 0x00);

	return;
}

void SFP_I2C_read_regs(void)
{
	uint8_t rx_data;

	rx_data = SFP_I2C_CTRL(I2C_SFP_CMD_INPUT, I2C_READ, 0);
	UART_polled_tx_string(&g_uart0, (uint8_t *)"\n\r SFP-INPUT-REG: ");
	UART_polled_tx_string(&g_uart0, itoa(rx_data));
	rx_data = SFP_I2C_CTRL(I2C_SFP_CMD_OUTPUT, I2C_READ, 0);
	UART_polled_tx_string(&g_uart0, (uint8_t *)"\n\r SFP-OUTPUT-REG: ");
	UART_polled_tx_string(&g_uart0, itoa(rx_data));
	rx_data = SFP_I2C_CTRL(I2C_SFP_CMD_POLARITY, I2C_READ, 0);
	UART_polled_tx_string(&g_uart0, (uint8_t *)"\n\r SFP-POLARITY-REG: ");
	UART_polled_tx_string(&g_uart0, itoa(rx_data));
	rx_data = SFP_I2C_CTRL(I2C_SFP_CMD_CONF, I2C_READ, 0);
	UART_polled_tx_string(&g_uart0, (uint8_t *)"\n\r SFP-CONF-REG: ");
	UART_polled_tx_string(&g_uart0, itoa(rx_data));
	UART_polled_tx_string(&g_uart0, (uint8_t *)"\n\r...\n\r");

	return;
}


/*
 * Send a byte by write to register 8 of APB_SLAVE
 * The whole frame consists of 8 words including the k-value,
 * two counter, the byte to send, the digits 0 to F and some hex-text.
 */
void Lane_send_byte(uint8_t b)
{
	uint32_t * pSFP_Regs;

	pSFP_Regs = (uint32_t *)APB_SLAVE_ADDRESS + 8;
	*pSFP_Regs = b;

	return;
}

void Lane_print_rx(void)
{
	 uint32_t buff[12];
	 uint32_t * p = (uint32_t *)APB_SLAVE_ADDRESS;
	 int i;

	 for(i = 0; i < 12; i++)
		 buff[i] = *(p + i);

	 /*
	  * print otut reg 1 to 3; the counter and the receives byte
	  */
	 for(i = 1; i < 4; i++)
		 UART_polled_tx_string(&g_uart0, itoa(buff[i]));

	 UART_polled_tx_string(&g_uart0, (uint8_t *)"\n\r");

	return;
}

